@@ -233,22 +233,23 @@ class AddGraphRelationships(Neo4jBase):
             else destination_column
         )
 
+        # ✅ Use two MATCH clauses → no Cartesian product warning
         query = (
-            f"MATCH (a:{source_label} {{{source_key}: {source_value}}}), "
-            f"(b:{destination_label} {{{destination_key}: {target_value}}}) "
+            f"MATCH (a:{source_label} {{{source_key}: {source_value}}})\n"
+            f"MATCH (b:{destination_label} {{{destination_key}: {target_value}}})\n"
             f"MERGE (a)-[r:{relationship_type}]->(b)"
         )
 
         if properties:
             set_clauses = [
-                f"r.{prop.replace(' ', '').replace('[', '_').replace(']', '_')}"
-                f"= {self._generate_property_string(value)}"
+                f"r.{prop.replace(' ', '').replace('[', '_').replace(']', '_')}="
+                f"{self._generate_property_string(value)}"
                 for prop, value in properties.items()
             ]
             if set_clauses:
                 query += " SET " + ", ".join(set_clauses)
 
-        self.logger.debug(f"Generated query: {query}")
+        self.logger.debug("Generated query: %s", query)
         return query
 
     def _id_key_for_label(self, label: str) -> str:
@@ -277,12 +278,10 @@ class AddGraphRelationships(Neo4jBase):
         - Picks values by the expected keys inferred from node labels.
         Returns: (normalized_df, src_col_name, dst_col_name)
         """
-        # Make sure we have the columns we need
         if "ID_1" not in df.columns or "ID_2" not in df.columns:
-            # Return an empty normalized frame with inferred column names
             src_key = self._id_key_for_label(source_label)
             dst_key = self._id_key_for_label(destination_label)
-            return pd.DataFrame(columns=[src_key, dst_key, "Evidence"]), src_key, dst_key
+            return pd.DataFrame(columns=[src_key + "_src", dst_key + "_dst", "Evidence"]), src_key + "_src", dst_key + "_dst"
 
         def to_dict(x):
             try:
@@ -290,28 +289,67 @@ class AddGraphRelationships(Neo4jBase):
             except Exception:
                 return {}
 
-        src_key = self._id_key_for_label(source_label)       # e.g., "CID" or "GeneSymbol"
-        dst_key = self._id_key_for_label(destination_label)  # e.g., "CID"
+        src_key = self._id_key_for_label(source_label)      # "CID" or "GeneSymbol"
+        dst_key = self._id_key_for_label(destination_label) # "CID" or "GeneSymbol"
 
         d1 = df["ID_1"].apply(to_dict)
         d2 = df["ID_2"].apply(to_dict)
 
-        # Extract, allowing the pair to be in either ID_1 or ID_2
         src_vals = d1.apply(lambda d: d.get(src_key)).fillna(d2.apply(lambda d: d.get(src_key)))
         dst_vals = d2.apply(lambda d: d.get(dst_key)).fillna(d1.apply(lambda d: d.get(dst_key)))
 
+        src_col = f"{src_key}_src"
+        dst_col = f"{dst_key}_dst"
+
         out = pd.DataFrame({
-            src_key: src_vals.astype(str),
-            dst_key: dst_vals.astype(str),
+            src_col: src_vals.astype(str),
+            dst_col: dst_vals.astype(str),
         })
 
         if "Evidence" in df.columns:
             out["Evidence"] = df["Evidence"]
 
-        # Drop incomplete rows
-        out = out.dropna(subset=[src_key, dst_key], how="any")
+        out = out.dropna(subset=[src_col, dst_col], how="any")
 
-        return out, src_key, dst_key
+        return out, src_col, dst_col
+
+
+    def _parse_listish(self, value):
+        """
+        Return a list of string targets from a variety of formats:
+        - "[123, 456]" or "['123','456']"  -> ["123","456"]
+        - "123,456" or "123 ; 456"         -> ["123","456"]
+        - "12345" (scalar)                 -> ["12345"]
+        - NaN / empty                      -> []
+        """
+        if value is None or (isinstance(value, float) and np.isnan(value)):
+            return []
+
+        # Work with strings thereafter
+        s = str(value).strip()
+        if not s or s.lower() in {"nan", "none"}:
+            return []
+
+        # Looks like a Python/JSON list?
+        if (s.startswith("[") and s.endswith("]")) or (s.startswith("(") and s.endswith(")")):
+            try:
+                v = ast.literal_eval(s)
+                if isinstance(v, (list, tuple, set)):
+                    return [str(x).strip() for x in v if str(x).strip()]
+                # If it’s a scalar inside the brackets for some reason
+                return [str(v)]
+            except Exception:
+                pass  # fall through to split heuristics
+
+        # Comma/semicolon separated?
+        if ("," in s) or (";" in s) or ("|" in s):
+            # Normalize multiple separators
+            for sep in [";", "|"]:
+                s = s.replace(sep, ",")
+            return [t.strip() for t in s.split(",") if t.strip()]
+
+        # Otherwise treat as single value
+        return [s]
 
 
     def generate_cypher_queries_from_file(
@@ -347,9 +385,11 @@ class AddGraphRelationships(Neo4jBase):
             "Similar CIDs": "CompoundID", "Target Accession": "ProteinRefSeqAccession",
             "geneid": "GeneID", "target_geneid": "GeneID", "cid": "CompoundID",     "GeneSymbol": "GeneSymbol",
             "genesymbol": "GeneSymbol", "CID": "CompoundID", "cid": "CompoundID",
-            "LinkID": "CompoundID", "linkid": "CompoundID", "ID": "CompoundID"
+            "LinkID": "CompoundID", "linkid": "CompoundID", "ID": "CompoundID",
+            "CID_src": "CompoundID", "CID_dst": "CompoundID", "GeneSymbol_src": "GeneSymbol",
+            "GeneSymbol_dst": "GeneSymbol", "AID_src": "AssayID", "AID_dst": "AssayID",
         }
-        self.logger.info(f"Reading data from CSV file: {file_path}")
+        # self.logger.info(f"Reading data from CSV file: {file_path}")
 
         # Read as strings to avoid dtype surprises; treat "__nan__" as NA
         df = pd.read_csv(
@@ -360,7 +400,8 @@ class AddGraphRelationships(Neo4jBase):
             engine="python",           # tolerant to weird quoting/newlines
             on_bad_lines="skip",       # skip broken Evidence rows
             quotechar='"',
-            escapechar='\\'
+            escapechar='\\',
+            index_col=False
         )
 
         df.columns = self._normalize_headers(df.columns.tolist())
@@ -371,29 +412,76 @@ class AddGraphRelationships(Neo4jBase):
             return
 
         # --- NEW unified branch for CO_OCCURS_IN_LITERATURE ---
-        if rel_type == "CO_OCCURS_IN_LITERATURE":
+        if rel_type in {
+            "CO_OCCURS_IN_LITERATURE",
+            "Compound_Gene_CoOccurrence",
+            "Compound_Compound_CoOccurrence",
+        }:
             # Normalize (works for both CID↔CID and GeneSymbol↔CID)
             df, source_column, destination_column = self._normalize_cooccurrence(df, source_label, destination_label)
 
             if df.empty:
                 self.logger.error("CSV %s contains no valid co-occurrence rows after normalization.", file_path)
                 return
+        
+        # --- SPECIAL CASE for compound transformation (substrate → metabolite) ---
+        if rel_type in {"Compound_Transformation", "METABOLIZED_TO", "TRANSFORMS_TO"}:
+            if {"substratecid", "metabolitecid"}.issubset(df.columns):
+                source_column = "substratecid"
+                destination_column = "metabolitecid"
+            else:
+                self.logger.error(
+                    "Compound_Transformation loader: expected 'substratecid' "
+                    "and 'metabolitecid' in %s", file_path
+                )
+                return
 
         else:
-            src_candidates = self._id_candidates_for_label(source_label)
-            dst_candidates = self._id_candidates_for_label(destination_label)
+            # --- SPECIAL CASE for similarity relationship ---
+            if rel_type in {"IS_SIMILAR_TO", "Compound_Similarity", "COMPOUND_SIMILARITY"}:
+                # Source: a single CID; Destination: list of similar CIDs
+                # Prefer canonical list column names when present
+                list_col_candidates = ["Similar CIDs", "Similar_CIDs", "similar_cids", "similarCIDs", "similar_ids"]
+                src_candidates = ["CID", "cid", "CompoundID", "Compound ID", "LinkID", "ID"]
+                source_column = self._pick_id_column(df, src_candidates)
 
-            source_column = self._pick_id_column(df, src_candidates)
-            destination_column = self._pick_id_column(df, dst_candidates)
+                # Find the first existing list column
+                destination_column = None
+                for cand in list_col_candidates:
+                    if cand in df.columns:
+                        destination_column = cand
+                        break
 
-            if not source_column or not destination_column:
-                self.logger.error("Could not detect ID columns for %s → %s in %s", source_label, destination_label, file_path)
-                return
+                if not source_column or not destination_column:
+                    self.logger.error(
+                        "Similarity loader: could not find required columns in %s "
+                        "(source one of %s, destination one of %s).",
+                        file_path, src_candidates, list_col_candidates
+                    )
+                    return
+
+                df = df.dropna(subset=[source_column], how="any")
+                if df.empty:
+                    self.logger.error("CSV %s contains no valid rows after filtering (source).", file_path)
+                    return
+
+            else:
+                # Generic path
+                src_candidates = self._id_candidates_for_label(source_label)
+                dst_candidates = self._id_candidates_for_label(destination_label)
+
+                source_column = self._pick_id_column(df, src_candidates)
+                destination_column = self._pick_id_column(df, dst_candidates)
+
+                if not source_column or not destination_column:
+                    self.logger.error("Could not detect ID columns for %s → %s in %s", source_label, destination_label, file_path)
+                    return
+
+                df = df.dropna(subset=[source_column, destination_column], how="any")
+                if df.empty:
+                    self.logger.error("CSV %s contains no valid rows after filtering.", file_path)
+                    return
 
-            df = df.dropna(subset=[source_column, destination_column], how="any")
-            if df.empty:
-                self.logger.error("CSV %s contains no valid rows after filtering.", file_path)
-                return
 
 
         # Only drop NA on rel_type_column if present in this file
@@ -416,12 +504,12 @@ class AddGraphRelationships(Neo4jBase):
                 )
             rel_type_column = None
 
-        # (Optional) write a small preview for debugging
-        try:
-            df.head(50).to_csv("Data/test.csv", index=False)
-            self.logger.info("Wrote preview to Data/test.csv (first 50 rows).")
-        except Exception as e:
-            self.logger.debug("Could not write Data/test.csv: %s", e)
+        # # (Optional) write a small preview for debugging
+        # try:
+        #     df.head(50).to_csv("Data/test.csv", index=False)
+        #     self.logger.info("Wrote preview to Data/test.csv (first 50 rows).")
+        # except Exception as e:
+        #     self.logger.debug("Could not write Data/test.csv: %s", e)
 
         for _, row in df.iterrows():
             src_raw = row[source_column]
@@ -435,10 +523,15 @@ class AddGraphRelationships(Neo4jBase):
             )
 
             if rel_type == "IS_SIMILAR_TO":
-                targets = ast.literal_eval(dst_raw)
+                targets = self._parse_listish(row.get(destination_column))
+                if not targets:
+                    continue
                 for target in targets:
+                    # skip degenerate self-links early (we also have a cleanup method)
+                    if str(target) == str(src_raw):
+                        continue
                     yield self._generate_query(
-                        src_raw, target, relationship_type, properties,
+                        str(src_raw), str(target), relationship_type, properties,
                         source_label, destination_label, standard_id,
                         source_column, destination_column
                     )
